--------------------------------------------------------------------------------
--  avatar.lua is part of ZS Shaiya Bot.
-- 
--  ZS Shaiya Bot is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
-- 
--  ZS Shaiya Bot is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
-- 
--  You should have received a copy of the GNU General Public License
--  along with ZS Shaiya Bot.  If not, see <http://www.gnu.org/licenses/>.
-- 
--  Copyright  2009 ZS Shaiya Bot
--------------------------------------------------------------------------------

-- default options.
avatar_default_options = {
  rest_enabled=true,
  target_enabled=true,
  rotate_view=true,
  unjam_enabled=true,
  jumping_enabled=true,
  pickup_enabled=true,
  flee_enabled=true,
  move_fast_enabled=false,
  displace_enabled=false,
  waypoint_enabled=false,
  cancel_requests_enabled=true,
  follow_party_enabled=false,
  fight_target_enabled=true,
  quick_fight_enabled=true,
  flee_hp=0.3,
  pickup_limit=2,
  jam_threshhold=5,
  --stuck_move_tolerance_min=0.0,
  stuck_move_tolerance_max=0.5,
  --boundary_range=12,
  --jump_move_tolerance_min=0.0,
  jump_move_tolerance_max=0.7,
  --ks_move_tolerance_min=0.15,
  hp_sit_multiplier=0.7,
  mp_sit_multiplier=0.0,
  sp_sit_multiplier=0.0,
  hp_sit_offset=100,
  mp_sit_offset=100,
  sp_sit_offset=100,
  max_fighting_delay=45,
  max_switch_targets=5,
  target_displaced_limit=1.0,
  target_difficulty_color_min="cyan",
  target_difficulty_color_max="green",
  interval_jump=2000,
  cannot_attack_delay=1000,
  move_flee_delay=12,
  look_around_arc=360,
  look_around_interval=90,
  target_undamaged_timeout=20,
  waypoint_count=2,
  waypoint_level_spread_lower=4,
  waypoint_level_spread_upper=0,
  rest_break_move=4,
}

-- Field Values.
USER_IDLE          = 0
USER_ATTACKING     = 1
USER_SITTING       = 3
USER_UNDER_ATTACK  = 5
USER_CANNOT_ATTACK = 6
USER_FLEE = 7
USER_DIED = 9

ATTACK_START  = 0
ATTACK_NORMAL = 1
ATTACK_END    = 2

MOVEMENT_SLOWEST = 0
MOVEMENT_SLOWER  = 1
MOVEMENT_NORMAL  = 2
MOVEMENT_FAST    = 3
MOVEMENT_STOP    = 255

SKILL_FAILURE = 0
SKILL_SUCCESS = 1

-- Timing Variables.
ACTION_DELAY         = 1000
ATTACK_DELAY         = 2000
SITTING_DELAY        = 3000
SIT_HIT_DELAY        = 250
SIT_CYCLE_DELAY      = 250
PICKUP_DELAY         = 750
CHECK_HEAL_DELAY     = 100
--ROTATE_90_DELAY      = 1000
MOVE_CHAR_AWAY_DELAY = 1250
JUMP_DELAY           = 1500
FLIP_DELAY           = 1500
TARGET_SEARCH_DELAY  = 250
BREAK_ACTION_DELAY   = 1000
--CAMERA_SPIN_SPEED_DELAY = 6.65
CAMERA_ROTATE_360_DELAY = 2400
CAMERA_RESET_DELAY   = 1000

AVATAR_ROTATE_SPEED_DELAY = 12
--MOVE_FLEE_DELAY = 15000
DOUBLE_TAP_DELAY     = 100
AVATAR_DISPLACE      = 2

--CANNOT_ATTACK_DELAY  = 5000;

--INTERVAL_JUMP   = 1500;
--INTERVAL_POTION = 500;

-- constructor.
-- @param obj The Avatar object.
-- @param options The avatar options, they will overwrite any default values.
Avatar = class(
  function(obj)
    obj.options = avatar_default_options

    -- Init variables.
    --self.closest_party_member_index = nil
    obj.closest_party_member_coords = nil
    obj.status = USER_IDLE
    obj.jam_likely = 0
    obj.switched_targets = 0
    --obj.damage_dealt = 0
    --obj.damage_dealt_prev = 0
    obj.pickup_count = 0
    -- waypoint data
    obj.target_tries = 0
    obj.waypoint = 0
    obj.waypoint_mob_index = 0

    obj.action = {}
    obj.potion = {}
    obj.buff = {}
    obj.skill = {}
    
    obj.name = Hook:get_avi_name()

    --debug_message("avatar_name: " .. obj.name)
    if (obj.name == 0) then
      --return AVATAR_ERROR_LOADING
    end

    obj:refresh()
    --registerTimer("avatar_slow_refresh", g_cfg.avatar_slow_refresh_interval, Avatar.slow_refresh, obj)
    registerTimer("follow_party", g_cfg.follow_party_interval, Avatar.follow_closest_party, obj)
  end
)


MINIMAP_REAL_COORD_FACTOR = 4.18


function Avatar:follow_closest_party()
    if (not self.options.follow_party_enabled) then
        return
    end
    local l_rel_coord = Gui:get_closest_party_member()
    --self.closest_party_member_coords = Gui:get_closest_party_member()
    --local l_rel_coord = self.closest_party_member_coords
    print("l_rel_coord")
    print_r(l_rel_coord)
    if (l_rel_coord == 0) then
        print("No party member found.")
        return
    end
    
    if (l_rel_coord.dist < 5 or l_rel_coord.dist > 55) then
        print("No need to move.")
        return
    end
    local l_quad_angle = get_quadrantal_angle(l_rel_coord)
    
    --local l_quad = get_quadrant(l_rel_ccord)
    --Avatar_actions:rotate_to(self, l_quad_angle, 1)
    local l_quad = get_quadrant(l_rel_coord)

    local l_dest = {x=(self.pos.x + (l_rel_coord.x * MINIMAP_REAL_COORD_FACTOR)),
        z=(self.pos.z - (l_rel_coord.y * MINIMAP_REAL_COORD_FACTOR))}

    --print_r(l_dest)
    Avatar_actions:move_to(self, l_dest)
end

function Avatar:set_class(class)
  self.class = class
end


function Avatar:set_options(options)
  -- Default values.
  self.options = merge(self.options, options)
end


function Avatar:set_shortcuts(shortcuts)
    Avatar_actions:init()
    for k,shortcut in pairs(shortcuts) do
        if (shortcut.type == "action") then
            --print_r(shortcut)
            --self.action[shortcut.mode] = g_key.shortcut[shortcut.bar][shortcut.slot]
            Avatar_actions:set_shortcut_key(shortcut)
        elseif (shortcut.type == "skill") then
            --print_r(shortcut)
            local l_key = g_key.shortcut[shortcut.bar][shortcut.slot]
            if (shortcut.mode == "attack") then
                table.insert(self.skill, Skill(shortcut.name, shortcut.level, l_key))
            elseif (shortcut.mode == "buff") then
                table.insert(self.buff, Skill(shortcut.name, shortcut.level, l_key))
            end
        elseif (shortcut.type == "potion") then
            local l_keys = {}
            for i = shortcut.slot_start, shortcut.slot_end do
                table.insert(l_keys, g_key.shortcut[shortcut.bar][i])
            end
            table.insert(self.potion, Potion(l_keys, shortcut.cooldown, shortcut.mode, shortcut.tolerance))
            --print("potions:")
            --print_r(self.potion)
        end
    end
end


-- refresh Refreshes the avatar's required variables.
function Avatar:refresh()
    self.hp = Hook:get_avi_data("hp")
    self.mp = Hook:get_avi_data("mp")
    self.sp = Hook:get_avi_data("sp")
    self.hp_max = Hook:get_avi_data("hp_max")
    self.mp_max = Hook:get_avi_data("mp_max")
    self.sp_max = Hook:get_avi_data("sp_max")
    --self.damage_dealt = Hook:get_avi_data("damage_dealt")
    self.action = Hook:get_avi_data("action")
    self.rotation = Hook:get_avi_rotation()
    self.level = Hook:get_avi_level()

    self.pos = Hook:get_avi_position()
    --debug_message("pos(" .. self.pos.x .. "," .. self.pos.y .. "," .. self.pos.z .. ")")

    if (self.prev_hp ~= nil) then
        if (self.status ~= USER_UNDER_ATTACK and self.status ~= USER_ATTACKING and self.hp < self.prev_hp) then
            print("avatar: under attack!")
            self.status = USER_UNDER_ATTACK
            debug_message("hp: " .. self.hp)
        end
        if (self.hp <= (self.hp_max * self.options.flee_hp)) then
            self.status = USER_FLEE
        end
    end
    if (self.hp <= 0) then
        debug_message("hp: " .. self.hp)
        self.status = USER_DIED
    end
    self.prev_hp = self.hp
end


-- Checks if the avatar can use any buffs,
--    in order of importance, and does so if it can.
-- @param buffs The table of buffs.
function Avatar:check_use_buffs(buffs)
  --debug_message("check_use_buffs()")
  for key,value in ipairs(self.buff) do
    self:check_use_skill(value)
  end
end


-- Checks if the avatar needs to use any potion, and does so
--    if they do.
function Avatar:check_use_potions()
    --debug_message("check_use_potions()")
    if (self.potion ~= nil and
            (self.action == AVATAR_ACTION_STANDING or self.action == AVATAR_ACTION_MOVING)) then
        for i,pot in pairs(self.potion) do
          if (pot:ready(self)) then
            pot:use()
            debug_message(sprintf("potion type %s used.", i))
          end
        end
    end
end


-- Check if the avatar can use a skill, and uses it if it's
--    available.
-- @param skill The skill object, that is to be checked, and used.
-- @return true When the skill has been used, false otherwise.
function Avatar:check_use_skill(skill)
  --debug_message("Avatar:check_use_skill()")
  local l_ready = skill:available(self.mp, self.sp)
  local l_used
  if (l_ready) then
    l_used = skill:use()
  end
  return l_used
end


-- checks a bar skill and uses it if it lower than the threshold.
-- @param bar The bar your wish to check, values are "hp", "mp", "sp".
-- @param tolerance If the bar goes lower than the point then execute skill (in decimal format).
-- @param skill The skill you wish to use.
function Avatar:check_use_bar_skill(bar, tolerance, skill)
    debug_message("Avatar: check_use_bar_skill()")
    local l_bar_lvl = g_gui:get_user_status(bar)
    if (l_bar_lvl <= tolerance) then
        debug_message(sprintf("l_bar_lvl: %d", l_bar_lvl))
        Avatar_actions:target_self()
        self:check_use_skill(skill)
    end
end


-- Checks the party bars to see who needs healing.
-- It will prioritize who needs healing most then use it.
function Avatar:check_use_heal_party(skill_info)
  debug_message("Avatar: check_use_heal_party()")
  -- prioritise who needs healing.
  l_gui = get_gui()
  l_party_members_health = l_gui:get_party_status()
  local l_func_compare = function (a, b) return (a.value < b.value) end
  table.sort(l_party_members_health, l_func_compare)
  --print("\nl_party_members_health:")
  --aprint_r(l_party_members_health)
  local l_skill_result
  local l_skill
  for k, v in ipairs(l_party_members_health) do
    if (v.value > 0) then
      l_skill = nil
      for l, w in ipairs(skill_info) do
        if (v.value <= w.tolerance) then
          debug_message("skill.tolerance: " .. w.tolerance)
          l_skill = w.skill
          break
        end
      end
      if (l_skill == nil) then
        break
      end
      debug_message("member_index: " .. v.index)
      debug_message("bar_value: " .. v.value)
      Avatar_actions:target_party_member(v.index)
      l_skill_result = self:check_use_skill(l_skill)
      --l_skill_result = SKILL_USED
      if (l_skill_result == SKILL_USED ) then
        break
      end
    end
  end
  --debug_message("skill: " .. l_skill)
  yrest(CHECK_HEAL_DELAY)
end


-- Checks if the avatar is jammed.
function Avatar:check_obstacle()
  --debug_message("check_obstacle()")
  
    local l_data = {
        GUI_TEXT_MESSAGE_LAND_OBSTACLE,
        GUI_TEXT_MESSAGE_OBSTACLE,
    }
    local l_is_obstacle = get_gui():in_latest_text_msg(l_data)
    if (l_is_obstacle) then
        self:unjam()
    end
end


-- Finds the target, by scrolling through the monsters,
--    or by doing various checks.
-- @return true when a target is found, false if not
function Avatar:cycle_target()
  if (not self.options.target_enabled) then
    return
  end
  if (self.options.waypoint_enabled == true and self.target_tries == self.options.waypoint_count) then
    self.target_tries = 0
    self:move_next_waypoint()
  end
  debug_message("Avatar: cycle_target()")
  local max_switch = self.options.max_switch_targets
  --self.switched_targets = 0
  --printf("self.switched_targets: %d\n", self.switched_targets)
  --printf("self.max_switch_targets: %d\n", self.max_switch_targets)
  if (self.options.rotate_view) then
    local l_look_arc = self.options.look_around_arc
    local l_look_interval_deg = self.options.look_around_interval
    local l_look_interval = math.floor(l_look_arc / l_look_interval_deg)
    --local l_interval_deg = l_look_arc / l_look_interval
    for i=1, l_look_interval do
      if (self.options.quick_fight_enabled and
            self.status == USER_UNDER_ATTACK) then
        break
      end
      Avatar_actions:spin_camera(DIRECTION_LEFT, l_look_interval_deg)
    end
  end
  local l_color_min = target_name_color[self.options.target_difficulty_color_min]
  local l_color_max = target_name_color[self.options.target_difficulty_color_max]
  printf("target_color_min: %s\n", l_color_min)
  printf("target_color_max: %s\n", l_color_max)
  for i=0, max_switch do
    if (self.status == USER_DIED) then
        return false
    end
    --debug_message("new_target")
    Avatar_actions:target()

    local l_target_status = Gui:get_target_status()
    --printf("target_status=%d\n", l_target_status)
    if (l_target_status == target_status.full) then
      local l_color = Gui:get_target_difficulty_color()
      printf("target_color: %s\n", l_color)
      if (l_color >= l_color_min and l_color <= l_color_max) then
        self.target_tries = 0
        return true
      end
    end
    if (self.options.quick_fight_enabled and
            l_target_status ~= target_status.unknown and
            self.status == USER_UNDER_ATTACK) then
      return true
    end
  end
  self.status = USER_IDLE  
  self.target_tries = self.target_tries + 1
  return false
end


function Avatar:fight_start()
    if (self.options.quick_fight_enabled and self.status == USER_UNDER_ATTACK) then
        return
    end
    if(Avatar_class:battle_round_start()) then
        debug_message("Avatar: fight_start()")
        local l_bm_res = self:battle_moving()
        if (l_bm_res == avi_moving_results.kill_stealing) then
            print("anti-ks.")
            return false
        end
        Avatar_actions:reset_camera()
    end
end


-- Makes the avatar fight the target,
--    until it's dead, or the fighting timer expires.
-- @param obj_class The class object of the avatar.
-- @param obj_target The target object of the avatar.
-- @returns true if battle was successful, false otherwise.
function Avatar:fight_target()
    if (not self.options.fight_target_enabled) then
        return
    end

    debug_message("Avatar: fight_target()")
    --registerTimer("check_jump", self.options.interval_jump, check_use_jump)
    self.fighting_timer = 0
    self.status = USER_ATTACKING
    --self.jam_likely = 0
    --self.no_jumping = false

    local max_fighting_delay = self.options.max_fighting_delay * 1000
    local l_no_dmg = self.options.target_undamaged_timeout  * 1000
    --debug_message(sprintf("avatar.fighting_timer: %d\n", self.fighting_timer));
    local l_gui = get_gui()
    --repeat
    local l_target_color = Gui:get_target_difficulty_color()
    local l_target_status = Gui:get_target_status()
    --while(not(l_target_color == target_name_color.unknown or l_target_color == target_name_color.white)) do
    while(l_target_color ~= target_name_color.unknown or l_target_status ~= target_status.unknown) do
    --while(Hook:get_has_target() == 1) do
        if (self.status == USER_DIED) then
            return false
        end
        if (self:check_use_flee()) then
            break
        end
        
        l_target_status = Gui:get_target_status()
        if (self.fighting_timer >= max_fighting_delay) then
            debug_message("avatar - fight timed out.")
            return false
        elseif (self.fighting_timer >= l_no_dmg and l_target_status == target_status.full) then
            --Avatar_actions:target_self()
            debug_message("avatar - no hurting")
            return false
        end

        local l_res = Avatar_class:battle_round()
        Avatar_actions:reset_camera()
        if (l_res == SKILL_WRONG_TARGET) then
            debug_message("avatar - wrong target.")
            return false
        --[[elseif (l_res == SKILL_CANNOT_ATTACK) then
          debug_message("avatar - cannot attack.")
          self.jam_likely = 0]]
        end

        local l_bm_res = self:battle_moving()
        if (l_bm_res == avi_moving_results.kill_stealing) then
            print("avatar - anti-ks.")
            return false
        end

        self:displace()
        self:check_obstacle()
        --debug_message(sprintf("avatar.damage_dealt=%d", self.damage_dealt))
        coroutine.yield()
        l_target_color = Gui:get_target_difficulty_color()
		l_target_status = Gui:get_target_status()
    end
    --until (Gui:get_target_difficulty_color() == target_name_color.unknown)
    Avatar_actions:target_self()
    --unregisterTimer("check_jump")
    debug_message("avi - target no hp")
    if (self.status ~= USER_FLEE) then
		self.status = USER_IDLE
	end
    yrest(g_cfg.max_round_trip)
    return true
end

function Avatar:battle_moving()
    local l_result
    while (self.action == AVATAR_ACTION_MOVING) do
        l_result = self:check_moving(true)
        if (l_result ~= moving) then
            return l_result
        end
    end
end


avi_moving_results = {
    moving=0,
    not_moving=1,
    kill_stealing=2,
}
    
    
function Avatar:check_moving(check_kill_steal)
    -- KS prevention, break attacked mobs that your moving towards.
    -- Not as effective because after attacks delay periods.
    --self.prev_pos = Hook:get_avi_position()
    self.action = Hook:get_avi_data("action")
    local l_prev_pos
    local l_target_status
    --while (self.action == AVATAR_ACTION_MOVING) do
        l_prev_pos = Hook:get_avi_position()
        yrest(g_cfg.avatar_slow_refresh_interval)
        self.action = Hook:get_avi_data("action")
        if (self.action ~= AVATAR_ACTION_MOVING) then
            return avi_moving_results.not_moving
        end
        self.pos = Hook:get_avi_position()
        local l_dist_avi_moved = distance_3d_point(l_prev_pos, self.pos)
        
        debug_message(sprintf("l_dist_avi_moved: %f", l_dist_avi_moved))
        --print("avatar: running to target")
        -- Check unjam
        if (self.options.unjam_enabled and
                --l_dist_avi_moved >= self.options.stuck_move_tolerance_min and
                l_dist_avi_moved <= self.options.stuck_move_tolerance_max
                ) then
            --[[if (self.action ~= AVATAR_ACTION_MOVING) then
                break
            end]]
            self:unjam()
        --end
        --self:check_unjam()
        --self:check_use_jump()

        -- Check Jump
        elseif (self.options.jumping_enabled and
                --l_dist_avi_moved >= self.options.jump_move_tolerance_min and
                l_dist_avi_moved <= self.options.jump_move_tolerance_max) then
            --debug_message(sprintf("l_dist_avi_moved: %f", l_dist_avi_moved))
            Avatar_actions:jump()
        --end        
        
        -- Check KS
        elseif (check_kill_steal == true) then
            l_target_status = Gui:get_target_status()
            print("l_target_status: ", l_target_status)
            if (l_target_status == target_status.damaged
                    --l_dist_avi_moved >= self.options.ks_move_tolerance_min
                    ) then
                debug_message("avatar: kill steal prevention.")
                Avatar_actions:target_self()
                return avi_moving_results.kill_stealing
            end
        end
    --end
end


-- Gets the avatar to pickup items.
-- @return nothing.
function Avatar:pickup_items()
  debug_message("Avatar: pickup_items()")
  self.pickup_count = (self.pickup_count + self.options["pickup_limit"])
  if (not self.options.pickup_enabled) then
    return
  elseif (self.status == USER_IDLE) then
    local l_gui = get_gui()
    local l_pickup = 0
    self.status = USER_LOOTING
    for i = (self.pickup_count - 1), 0, -1 do
      if(self.status ~= USER_LOOTING) then
        l_pickup = i
        debug_message("avatar - pickup break.")
        break
      --[[elseif (l_gui:in_latest_text_msg(GUI_TEXT_MESSAGE_INVENTORY_FULL)) then
        debug_message("avatar - insufficient inventory.")
        self.options.pickup_limit = 1
        break]]
      end
      Avatar_actions:pickup()
    end
    self.pickup_count = l_pickup
    --self:break_action();
  end
end


-- Checks if resting is required, and does so if required.
-- @return nothing.
-- TODO - Somehow we need to differentiate between monster damage,
--  and magic damage effects, so the avi doesn't think he's being attacked by
--  a mob when it is from magic effects.
function Avatar:check_use_rest()
  if (self.options.rest_enabled == false) then
    return
  end
  debug_message("Avatar: check_use_rest()")

  --self.sitting = 0
        
  if (self.hp <= (self.hp_max * self.options.hp_sit_multiplier) or
      self.mp <= (self.mp_max * self.options.mp_sit_multiplier) or
      self.sp <= (self.sp_max * self.options.sp_sit_multiplier)) then
    --[[if (self.options.pickup_enabled) then
      self:break_action()
    end]]
    print("avatar - moving")
    local l_start_time = os.time()
    while(self.action == AVATAR_ACTION_MOVING) do
        l_elapsed_time = os.difftime(os.time() - l_start_time)
        --print("l_elapsed_time: ", l_elapsed_time)
        if (l_elapsed_time >= self.options.rest_break_move) then
            print("avatar - moving cancelled to rest.")
            self:break_action()
        end
        --coroutine.yield()
        yrest(g_fps_delay)
    end
    yrest(g_cfg.max_round_trip)
      --self.status = USER_SITTING;
    Avatar_actions:sit()
    debug_message("avatar - sitting")
    while(self.hp <= (self.hp_max - self.options.hp_sit_offset) or
            self.mp <= (self.mp_max - self.options.mp_sit_offset) or
            self.sp <= (self.sp_max - self.options.sp_sit_offset)) do
      --debug_message(sprintf("avatar.sitting: %d\n", self.sitting))
        if (self.status == USER_DIED) then
            return false
        end
        if(self.action == AVATAR_ACTION_STANDING or
                self.action == AVATAR_ACTION_STAND_UP) then
            debug_message("avatar - attacked, standing")
            self.status = USER_UNDER_ATTACK
            --yrest(SIT_HIT_DELAY)
            yrest(g_fps_delay)
            return
        end
      yrest(SIT_CYCLE_DELAY)
      --coroutine.yield()
    end
    Avatar_actions:sit()
    self.status = USER_IDLE
    debug_message("avatar - standing")
  end
end


function Avatar:check_died()
    if (self.status == USER_DIED) then
        self:died()
        self:move_next_waypoint()
    end
end


-- Unjams the avatar by moving it away from the obstacle.
function Avatar:unjam()
  debug_message("Avatar: unjam()")
  Avatar_actions:reset_camera()
  yrest(CAMERA_RESET_DELAY)
  Avatar_actions:flip_backwards()
  -- Rotate Char either left or right.
  local l_choice = math.random(2)
  if (l_choice == 1) then
    l_choice = DIRECTION_LEFT
  else
    l_choice = DIRECTION_RIGHT
  end
  Avatar_actions:spin(l_choice, 100)
  Avatar_actions:move(DIRECTION_UP, MOVE_CHAR_AWAY_DELAY)
  yrest(g_cfg.max_round_trip)
end


function Avatar:displace(amount)
  if (self.options["displace_enabled"] ~= true) then
    return
  end
  local l_deg_cur = get_quadrantal_angle(self.rotation)
  l_rot_rad = math.rad(l_deg_cur)
  debug_message("l_deg_cur: " .. l_deg_cur)
  --local l_displace = 10
  local l_new_x = amount * math.cos(l_rot_rad)
  local l_new_z = amount * math.sin(l_rot_rad)
  debug_message("l_new_x: " .. l_new_x .. ", l_new_z: " .. l_new_z)
  self.pos.x = self.pos.x + l_new_x
  self.pos.z = self.pos.z + l_new_z
end


function Avatar:check_use_flee()
    if (self.options.flee_enabled and
            self.sitting == AVATAR_STANDING and
            self.status == USER_FLEE) then
        self:flee()
        return true
    end
    return false
end


function Avatar:move_next_waypoint()
    if (self.options.waypoint_enabled == false) then
        return
    end
    debug_message("move_next_waypoint()")
    
    
    g_waypoint = Xml_data:loadWaypoints(g_xml_file_waypoint)
    -- fix to reset camera.
    Avatar_actions:move(DIRECTION_UP, 1000)
    yrest(2000)

    local l_map = Hook:get_map_name()
    --print_r(g_waypoint[l_map])
    local l_waypoint_count = table_len(g_waypoint[l_map])

    local l_waypoint_distance
    local l_waypoint_shortest
    local l_waypoint_index
    local l_pos_cur = self.pos    
    local l_min_level = self.level - self.options.waypoint_level_spread_lower
    local l_max_level = self.level + self.options.waypoint_level_spread_upper

    -- return avatar to nearest waypoint.
    for i, wp_path in pairs(g_waypoint[l_map]) do
        --debug_message("waypoint_checked: "..i)
        --debug_message("called.")
        l_waypoint_distance = distance_2d_point({x=l_pos_cur.x, y=l_pos_cur.z}, wp_path)
        if (l_waypoint_shortest == nil or 
                l_waypoint_distance < l_waypoint_shortest) then
            l_waypoint_shortest = l_waypoint_distance
            l_waypoint_index = i
            self.waypoint_index_prev = i
            --debug_message("waypoint_prev_index: " .. self.waypoint_index_prev)
            --debug_message("closest waypoint: " .. l_waypoint_index .. ", distance: " .. l_waypoint_shortest)
        end
    end

   self.waypoint = g_waypoint[l_map][l_waypoint_index]
    if (self.waypoint == nil) then
        error("map: " .. l_map .. ", waypoint: " .. l_waypoint_index .. ", needs to be defined.")
    end

    while (true) do
        debug_message("moving to waypoint: " .. l_waypoint_index .. " - (" .. self.waypoint.x .. "," .. self.waypoint.z .. ")")
        --self:move_to(self.waypoint)
        Avatar_actions:move_to(self, self.waypoint)
        debug_message("waypoint: " .. l_waypoint_index .. " reached.")
        
        --debug_message("waypoint_index: " .. l_waypoint_index)
        --debug_message("waypoint_prev_index: " .. self.waypoint_index_prev)
        if (self.waypoint.level ~= nil and l_waypoint_index ~= self.waypoint_index_prev) then
            if (self.waypoint.level >= l_min_level) then
                 if (self.waypoint.level > l_max_level) then
                    --backtrack.
                    self.waypoint = g_waypoint[l_map][self.waypoint_index_prev]
                    self.waypoint_index_prev = l_waypoint_index
                 end
                 return
            end
        end

        -- select random child path.
        if (self.waypoint.links == nil) then
            error("map: " .. l_map .. ", waypoint: " .. l_waypoint_index .. ", no link waypoints.")
        end
        local l_link_size = table_len(self.waypoint.links)
        debug_message("l_link_size: " .. l_link_size)
        
        local l_temp_waypoint;

        local l_link_index;

        while(l_temp_waypoint == nil) do
            -- Choose the next waypoint, on several choices choose a random new one.
            debug_message("l_link_size: " .. l_link_size)
            if (l_link_size == 1) then
                l_link_index = 1;
            else
                local l_rand
                --math.randomseed(os.time())
                debug_message("self.waypoint_index_prev: " .. self.waypoint_index_prev)
                repeat
                    l_rand = math.random(1, l_link_size)
                    debug_message("l_rand: " .. l_rand)
                    debug_message("self.waypoint.links[l_rand]: " .. self.waypoint.links[l_rand])
                until (self.waypoint_index_prev ~= self.waypoint.links[l_rand])
                l_link_index = l_rand;
            end
            l_temp_waypoint_index = self.waypoint.links[l_link_index]
            l_temp_waypoint = g_waypoint[l_map][l_temp_waypoint_index]
            -- if the temp new waypoint is higher than max level, then search for another.
            if (l_temp_waypoint.level ~= nil) then
                debug_message("possible waypoint level: " .. l_temp_waypoint.level)
                -- Check that the mob level req is not too high.
                debug_message("l_waypoint_index: " .. l_waypoint_index)
                debug_message("l_temp_waypoint_index: " .. l_temp_waypoint_index)
                
                if(l_temp_waypoint.level > l_max_level) then
                    self.waypoint_index_prev = l_temp_waypoint_index
                    l_temp_waypoint = nil
                end
            end
            coroutine.yield()
        end

        self.waypoint_index_prev = l_waypoint_index
        debug_message("waypoint_prev_index: " .. self.waypoint_index_prev)

        --l_waypoint_index = self.waypoint.links[l_link_index]
        l_waypoint_index = l_temp_waypoint_index
        debug_message("waypoint: " .. l_waypoint_index)
        --self.waypoint = g_waypoint[l_map][l_waypoint_index]
        self.waypoint = l_temp_waypoint
    end
end


-- Adds to the fighting time.
-- @param time The time to be added.
function Avatar:add_fighting_time(p_time)
  if (self.status == USER_ATTACKING) then
    self.fighting_timer = self.fighting_timer + p_time
  end
end


function Avatar:flee()
    debug_message("Avatar: flee()")
    --self:break_action()
	
	keyboardPress(g_key.reset_camera)
	keyboardPress(g_key.flip_view)
    keyboardHold(g_key.move[DIRECTION_UP])
	keyboardPress(g_key.target_self)
    --Avatar_actions:spin(DIRECTION_LEFT, 180)
    
    --yrest(MOVE_FLEE_DELAY)
    local l_start_time = os.time()
    local l_elapsed_time = 0
    while(l_elapsed_time < self.options.move_flee_delay) do
        self:check_moving(false)
        coroutine.yield()
        l_elapsed_time = os.difftime(os.time() - l_start_time)
        print("l_elapsed_time: ", l_elapsed_time)
    end
    keyboardRelease(g_key.move[DIRECTION_UP])

    --Avatar_actions:move(DIRECTION_UP, MOVE_FLEE_DELAY)
    --self:check_moving()
    keyboardPress(g_key.target_self)
    --self:break_action()
    --self:check_use_rest()
	self.status = USER_FLEE
end


function Avatar:died()
    print("Avatar: died()")
    while(self.hp ~= self.hp_max) do
        coroutine.yield()
    end
end


-- Breaks the action of the avatar.
function Avatar:break_action()
    debug_message("Avatar: break_action()")
    Avatar_actions:flip_backwards()
end


function Avatar:check_cancel_player_requests()
    if (cancel_requests_enabled == false) then
        return
    end
    if (Gui:check_player_requests()) then
        print "avatar - player request detected, cancelling."
        -- Cancel the Request, and return Cursor to previous position.
        local cursor_pos = {x=(g_game_cfg.VIDEO.SIZE_X / 2) + 50,
                y=(g_game_cfg.VIDEO.SIZE_Y / 2) + 80}
        local l_prev_x, l_prev_y = mouseGetPos()
        mouseSetWin(g_window, cursor_pos.x, cursor_pos.y)
        mouseLClick()
        yrest(g_fps_delay)
        mouseSet(l_prev_x, l_prev_y)
    end
end

function Avatar:check_use_attack()
    --print("Avatar:check_use_attack()")
    --print("self.action: ", self.action)
    if (self.action == AVATAR_ACTION_ATTACKING) then
        --print("avatar - alrady attacking.")
        return
    end
    Avatar_actions:attack()
	print("avatar - attack pressed.")
end
