--------------------------------------------------------------------------------
--  avatar_actions.lua is part of ZS Shaiya Bot.
-- 
--  ZS Shaiya Bot is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
-- 
--  ZS Shaiya Bot is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
-- 
--  You should have received a copy of the GNU General Public License
--  along with ZS Shaiya Bot.  If not, see <http://www.gnu.org/licenses/>.
-- 
--  Copyright  2009 ZS Shaiya Bot
--------------------------------------------------------------------------------

Avatar_actions = class(
    function(obj)
        --obj.action = {}
    end
)

function Avatar_actions:init()
    self.action = {}
end


function Avatar_actions:set_shortcut_key(shortcut)
    --[[if (self.action == nil) then
        self.action = {}
    end]]
    self.action[shortcut.mode] = g_key.shortcut[shortcut.bar][shortcut.slot]
end


function Avatar_actions:move_to(avatar, pos)
    local l_tolerance = 4
    local l_spread = 4
    local l_correction_dist = 10
    --debug_message("pos (" .. math.floor(avatar.pos.x) .. "," .. math.floor(avatar.pos.z) .. ")")
    --debug_message("move_to (" .. pos.x .. "," .. pos.z .. ")")

    local l_distance = distance_2d_point({x=avatar.pos.x, y=avatar.pos.z}, {x=pos.x, y=pos.z})
    
    local l_pos_relative = {
        x=(pos.x - avatar.pos.x),
        y=(pos.z - avatar.pos.z),
    }    
    local l_deg = get_quadrantal_angle(l_pos_relative)
    
    local l_rotation_spread = l_spread
    self:rotate_to(avatar, l_deg, l_rotation_spread)

    local l_pos_new = {
        min_x=(pos.x - l_tolerance),
        max_x=(pos.x + l_tolerance),
        min_z=(pos.z - l_tolerance),
        max_z=(pos.z + l_tolerance),
    }
    local l_next_correction_dist = (l_distance - l_correction_dist)
    local l_correction_current = 1
    keyboardHold(g_key.move[DIRECTION_UP])
    while (true) do
        avatar.pos = Hook:get_avi_position()
        l_distance = distance_2d_point({x=avatar.pos.x, y=avatar.pos.z}, {x=pos.x, y=pos.z})
        if ((l_distance >= l_correction_dist) and (l_distance < l_next_correction_dist)) then
            l_pos_relative = {
                x=(pos.x - avatar.pos.x),
                y=(pos.z - avatar.pos.z),
            }
            l_deg = get_quadrantal_angle(l_pos_relative)
            self:rotate_to(avatar, l_deg, l_rotation_spread)
            l_next_correction_dist = (l_distance - l_correction_dist)
        end
        if (avatar.pos.x >= l_pos_new.min_x and 
            avatar.pos.x <= l_pos_new.max_x and
            avatar.pos.z >= l_pos_new.min_z and 
            avatar.pos.z <= l_pos_new.max_z)then
                break;
        end
        coroutine.yield()
    end
    keyboardRelease(g_key.move[DIRECTION_UP])
    debug_message("moved_to (" .. math.floor(avatar.pos.x) .. "," .. math.floor(avatar.pos.z) .. ")")
end


function Avatar_actions:rotate_to(avatar, degrees, tolerance)
    --local l_pos_cur = avatar:get_rotation(l_pos)
    local l_deg_cur = get_quadrantal_angle(avatar.rotation)
    --debug_message("deg_cur: " .. l_deg_cur)
    --debug_message("deg_new: " .. degrees)
    --local l_increment = 0.75
    local l_pos_new = {
        x=math.cos(math.rad(degrees)),
        y=math.sin(math.rad(degrees)),
    }
    --debug_message("l_x: " .. l_pos_new.x .. ", " .. "l_y: " .. l_pos_new.y)

    --local l_tolerance = math.rad(l_increment)
    local l_tolerance = math.rad(tolerance)
    local l_direction = get_rotate_direction(l_deg_cur, degrees)

    keyboardHold(g_key.rotate[l_direction])
    while (true) do
        avatar.rotation = Hook:get_avi_rotation()
        --avatar:spin(l_direction, l_increment)
        --l_pos_cur = avatar:get_rotation(l_pos)
        
        --debug_message("l_cx: " .. l_pos_cur.x .. ", " .. "l_cy: " .. l_pos_cur.y)
        if(math.abs(avatar.rotation.x - l_pos_new.x) <= l_tolerance and
                math.abs(avatar.rotation.y - l_pos_new.y) <= l_tolerance) then
            break
        end
        coroutine.yield()
    end
    keyboardRelease(g_key.rotate[l_direction])
end


function Avatar_actions:cannot_attack()
  yrest(g_cfg.cannot_attack_delay)
end


function Avatar_actions:move(direction, duration)
  keyboardHold(g_key.move[direction])
  yrest(duration)
  keyboardRelease(g_key.move[direction])
end


function Avatar_actions:flip_backwards()
  keyboardPress(g_key.move[DIRECTION_DOWN])
  yrest(DOUBLE_TAP_DELAY)
  keyboardPress(g_key.move[DIRECTION_DOWN])
  yrest(FLIP_DELAY)
end


-- Gets the avatar to jump towards the target.
function Avatar_actions:jump()
  keyboardPress(g_key.jump)
end


-- Makes the avatar attack the target.
function Avatar_actions:attack()
  if (self.action.attack == nil) then
    return
  end
  keyboardPress(self.action.attack)
  yrest(ATTACK_DELAY)
end


-- Makes the avatar sit.
function Avatar_actions:sit()
  keyboardPress(g_key.sit)
  yrest(SITTING_DELAY)
end


function Avatar_actions:spin(direction, angle)
  keyboardHold(g_key.rotate[direction])
  yrest(AVATAR_ROTATE_SPEED_DELAY * angle)
  keyboardRelease(g_key.rotate[direction])
end


function Avatar_actions:spin_camera(direction, angle)
  
  keyboardHold(g_key.look[direction])
  local l_rest = math.floor(CAMERA_ROTATE_360_DELAY * (angle / 360))
  --yrest(CAMERA_SPIN_SPEED_DELAY * angle)
  --print("l_rest: ", l_rest)
  yrest(l_rest)
  keyboardRelease(g_key.look[direction])
end


function Avatar_actions:reset_camera()
    keyboardPress(g_key.reset_camera)
end


function Avatar_actions:target_party_member(index)
  keyboardPress(g_key.target_party_member[index])
  yrest(g_fps_delay)
end


function Avatar_actions:target_self()
  keyboardPress(g_key.target_self)
  yrest(g_fps_delay)
end


-- Makes the avatar target.
function Avatar_actions:target()
  keyboardPress(g_key.target)
  yrest(g_fps_delay + TARGET_SEARCH_DELAY)
end


function Avatar_actions:pickup()
  --keyboardPress(key_pickup)
  keyboardPress(self.action.pickup)
  yrest(PICKUP_DELAY)
end
