--------------------------------------------------------------------------------
--  shortcut.lua is part of ZS Shaiya Bot.
-- 
--  ZS Shaiya Bot is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
-- 
--  ZS Shaiya Bot is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
-- 
--  You should have received a copy of the GNU General Public License
--  along with ZS Shaiya Bot.  If not, see <http://www.gnu.org/licenses/>.
-- 
--  Copyright  2009 ZS Shaiya Bot
--------------------------------------------------------------------------------

-- Shortcut this is a base class for the shortcuts.
-- shortcut_key Shortcut key of the Shortcut.
-- cooldown The time it takes to cooldown the shortcut, in seconds.

-- constructor.
-- @param shortcut_key Shortcut keyboard key value.
-- @param cooldown The time taken for the skill to be re-used, in seconds.
Shortcut = class(function(obj, shortcut_key, cooldown)
    obj.shortcut_key = shortcut_key
    obj.cooldown = cooldown + 1 -- take into account os.time() rounding.
    obj.timer = os.difftime(os.time() - obj.cooldown)
    --obj.timer = obj.cooldown;
  end
)


-- avaliable Checks if the shortcut is available.
-- @return true if timer has been exceeded, false otherwise.
function Shortcut:available()
  --print(self:debug());
  --local l_time_elapsed = os.time()
  local l_time_elapsed = os.difftime(os.time() - self.timer)
  if (l_time_elapsed >= self.cooldown) then
    return true;
  else
    return false;
  end
end

-- use Uses the shortcut.
function Shortcut:use()
  --self.timer = 0;
  self.timer = os.time()
  keyboardPress(self.shortcut_key);
  --self.debug()
end

-- debug Used for debugging information of the shortcut.
-- @return string containing debugging info.
function Shortcut:debug()
  return sprintf("[skill] - shortcut_key: %d" ..
    ", cooldown: %d, timer: %d\n",
    self.shortcut_key, self.cooldown, self.timer);
end

function Shortcut:get_time()
  return self.timer;
end
