--------------------------------------------------------------------------------
--  xml_data.lua is part of ZS Shaiya Bot.
-- 
--  ZS Shaiya Bot is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
-- 
--  ZS Shaiya Bot is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
-- 
--  You should have received a copy of the GNU General Public License
--  along with ZS Shaiya Bot.  If not, see <http://www.gnu.org/licenses/>.
-- 
--  Copyright  2009 ZS Shaiya Bot
--------------------------------------------------------------------------------

error_xml_file = "XML file not found, or could not be parsed."

Xml_data = class(
    function(obj)
    end
)


function Xml_data:loadWaypoints(file)
    --print("Xml_data:loadWaypoints()")

    local l_xml = xml.open(file)

    local l_map = {}

    local l_map_child_name
    local l_map_name
    local l_wp_name
    local l_wp_level
    local l_wp_links
    local l_xml_map = l_xml:getElements()
    for k, maps_child in pairs(l_xml_map) do
        local l_map_child = maps_child:getElements()
        for k, map_child in pairs(l_map_child) do
            l_map_child_name = map_child:getName() 
            --print(l_map_child_name)
            if (l_map_child_name == "name") then
                l_map_name = map_child:getValue()
                --print("map_name: " .. l_map_name)
                l_map[l_map_name] = {}
            elseif (l_map_child_name == "waypoint") then
                l_xml_waypoint = map_child:getElements("waypoint")
                l_wp_name = map_child:getElement("name"):getValue()
                l_map[l_map_name][l_wp_name] = {}
                l_map[l_map_name][l_wp_name].x = tonumber(map_child:getElement("position"):getElement("x"):getValue())
                l_map[l_map_name][l_wp_name].z = tonumber(map_child:getElement("position"):getElement("y"):getValue())
                if (map_child:getElement("level")) then
                    l_wp_level = map_child:getElement("level"):getValue()
                    l_map[l_map_name][l_wp_name].level = tonumber(l_wp_level)
                end
                if (map_child:getElement("links")) then
                    l_wp_links = map_child:getElement("links"):getElements()
                    l_map[l_map_name][l_wp_name].links = {}
                    for k, wp_link in pairs(l_wp_links) do
                        table.insert(l_map[l_map_name][l_wp_name].links, wp_link:getValue())
                        --debug_message("link[" .. k .. "]: " .. wp_link:getValue())
                    end
                end
            end
        end
    end
    --print_r(l_map)
    return l_map
end


function Xml_data:loadSkills(file)
    --print("Xml_data:loadSkills()")

    local l_xml = xml.open(file)

    local l_skill = {}

    local l_xml_skills = l_xml:getElements("skills")
    local l_skill_name
    local l_skill_children
    local l_skill_child_name
    local l_skill_level_index
    for k, skill in pairs(l_xml_skills) do
        --print_r(skill)
        --debug_message("skill_name: " .. l_skill_name)
        l_skill_children = skill:getElements("level")
        for k, skill_child in pairs(l_skill_children) do
            --print_r(skill_child)
            l_skill_child_name = skill_child:getName()
            if (l_skill_child_name == "name") then
                l_skill_name = skill_child:getValue()
                --debug_message("l_skill_name: " .. l_skill_name)
                l_skill[l_skill_name] = {}
            elseif (l_skill_child_name == "duration") then
                l_skill[l_skill_name].duration = tonumber(skill_child:getValue())
            elseif (l_skill_child_name == "level") then
                --print_r(skill_child)
                --debug_message("skill_child.index: " .. skill_child:getAttribute("index"))
                l_skill_level_index = tonumber(skill_child:getAttribute("index"))

                l_skill[l_skill_name][l_skill_level_index] = {
                    recharge=g_cfg.skill_cooldown,
                    cast=0,
                    mp=0,
                    sp=0,
                }

                if(skill_child:getAttribute("recharge") ~= nil) then
                    l_skill[l_skill_name][l_skill_level_index].recharge = tonumber(skill_child:getAttribute("recharge"))
                end
                if(skill_child:getAttribute("cast") ~= nil) then
                    l_skill[l_skill_name][l_skill_level_index].cast = tonumber(skill_child:getAttribute("cast"))
                end                
                if(skill_child:getAttribute("mp") ~= nil) then
                    l_skill[l_skill_name][l_skill_level_index].mp = tonumber(skill_child:getAttribute("mp"))
                end
                if(skill_child:getAttribute("sp") ~= nil) then
                    l_skill[l_skill_name][l_skill_level_index].sp = tonumber(skill_child:getAttribute("sp"))
                end
            end
        end
    end
    --print_r(l_skill)
    return l_skill
end


function Xml_data:loadChar(file)
    --debug_message("Xml_data:loadChar()")
    local l_xml = xml.open(file)

    local l_avatar = {}
    local l_xml_char = l_xml:getElements()
    
    for k, child in pairs(l_xml_char) do
        --print(child:getName())
        local l_name = child:getName()
        if (l_name == "shortcuts") then
            local l_xml_shortcuts = child:getElements()
            l_avatar.shortcuts = {}
            for k, shortcut in pairs(l_xml_shortcuts) do                
                local l_shortcut = {}
                l_shortcut.type = shortcut:getAttribute("type")
                l_shortcut.bar = tonumber(shortcut:getAttribute("bar"))
                l_shortcut.mode = shortcut:getAttribute("mode")
                if (shortcut:getAttribute("slot") ~= nil) then
                    l_shortcut.slot = tonumber(shortcut:getAttribute("slot"))
                    if (l_shortcut.slot == 0) then
                        l_shortcut.slot = 10
                    end
                end
                if (l_shortcut.type == "skill") then
                    l_shortcut.name = shortcut:getAttribute("name")
                    l_shortcut.level = tonumber(shortcut:getAttribute("level"))
                    --[[if (shortcut:getAttribute("wait") ~= nil) then
                        l_shortcut.wait = tonumber(shortcut:getAttribute("wait"))
                    else
                        l_shortcut.wait = g_cfg.attack_delay
                    end]]
                elseif (l_shortcut.type == "potion") then
                    if (shortcut:getAttribute("slot") ~= nil) then
                        l_shortcut.slot_start = shortcut:getAttribute("slot")
                        l_shortcut.slot_end = shortcut:getAttribute("slot")
                    else
                        l_shortcut.slot_start = shortcut:getAttribute("slot_start")
                        l_shortcut.slot_end = shortcut:getAttribute("slot_end")            
                    end
                    if (l_shortcut.slot_start == 0) then
                        l_shortcut.slot_start = 10
                    end
                    if (l_shortcut.slot_end == 0) then
                            l_shortcut.slot_end = 10
                    end

                    l_shortcut.tolerance = tonumber(shortcut:getAttribute("tolerance"))
                    if (shortcut:getAttribute("cooldown") ~= nil) then
                        l_shortcut.cooldown = tonumber(shortcut:getAttribute("cooldown"))
                    else
                        l_shortcut.cooldown = g_cfg.potion_cooldown
                    end
                end
                --print_r(l_shortcut)
                table.insert(l_avatar.shortcuts, l_shortcut)
            end
        elseif (l_name == "options") then
            local l_xml_options = child:getElements()
            l_avi_options = {}
            local l_avi_opt_name
            local l_avi_opt_type
            local l_avi_opt_value
            for k, option in pairs(l_xml_options) do
                --print_r(option)
                l_avi_opt_name = option:getAttribute("name")
                l_avi_options[l_avi_opt_name] = option:getAttribute("value")
            end
            l_avatar.options = l_avi_options
        end
        --print_r(child)
    end
    --print_r(l_avatar)
    return l_avatar
end


function Xml_data:loadConfig(file)
    --print("Xml_data:loadConfig()")

    local l_xml = xml.open(file)

    local l_cfg = {}
    local l_xml_elems = l_xml:getElements()

    local l_cfg_name
    local l_cfg_type
    for k, cfg in pairs(l_xml_elems) do
        l_cfg_name = cfg:getAttribute("name")
        l_cfg[l_cfg_name] = cfg:getValue()
    end
    --print_r(l_cfg)
    return l_cfg
end

