--------------------------------------------------------------------------------
--  functions.lua is part of ZS Shaiya Bot.
-- 
--  ZS Shaiya Bot is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
-- 
--  ZS Shaiya Bot is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
-- 
--  You should have received a copy of the GNU General Public License
--  along with ZS Shaiya Bot.  If not, see <http://www.gnu.org/licenses/>.
-- 
--  Copyright  2008 ZS Shaiya Bot
--------------------------------------------------------------------------------

function check_use_jump()
  g_avatar:check_use_jump();
end


function init_window(win)
  local l_win = findWindow(win);
  --win = 852182;
  debug_message("Shaiya Window process ID: " .. l_win);
  assert((l_win ~= 0), error_init)
  return l_win;
end


function init_process()
  --debug_message("Hook init()");
  --local l_proc = openProcess( findProcessByExe(g_game_exe) );
  local l_proc = openProcess(findProcessByWindow(get_win()));

  --debug_message(sprintf("Shaiya process ID: %s", l_proc));
  assert((l_proc ~= 0), error_init);
  return l_proc;
end


function init_hdc()
  g_hdc = openDC(get_win());
  local wx, wy, ww, wh = windowRect(get_win());
  debug_message("win_width: " .. ww .. ", win_height: " .. wh .. "\n");
end


function get_win()
  return g_window;
end


function get_process()
  return g_proc;
end


function get_hdc()
  return g_hdc;
end


function get_gui()
  return g_gui;
end


function get_target()
  return g_target;
end


function get_avatar()
  return g_avatar;
end


function force_close()
  print("force_close");
  keyboardHold(key.VK_LALT);
  rest(50);
  keyboardPress(key.VK_F4);
  rest(50);
  keyboardRelease(key.VK_LALT);
  stopPE();
end


--[[function write_logs()
  if (not g_cfg.write_logs) then
    return
  end
  print("write_logs()")
  local l_gui = get_gui();
  local l_log = g_game_log;
  local l_prev_msg = g_game_msg;
  g_game_msg = l_gui:get_latest_text_msg();
  local l_log_msg = g_game_msg;
  if (l_prev_msg ~= g_game_msg) then
    --debug_message("g_game_msg: " .. g_game_msg);
    local l_trade_index = g_game_msg:find(g_message_trade_uid, 1, true);
    if (l_trade_index ~= nil) then
      --debug_message(sprintf("l_trade_index: %d", l_trade_index));
      l_log = g_trade_log;
      l_log_msg = g_game_msg:sub((g_message_trade_uid:len() + 1));
    end
    local l_rec = get_dt_rec(l_log_msg);
    --l_log:write(l_rec);
    append_to_file(l_log, l_rec);
    --debug_message(l_log_msg);
  end
end]]


function save_trade_message(proc, channel)
    if (not g_cfg.write_logs) then
        return
    end
    --debug_message(sprintf("l_address: %d", l_address))
    local l_message = Hook:get_latest_text_msg(channel.name, proc)
    --local l_message = Gui:get_latest_text_msg()
    if (l_message ~= nil and l_message ~= channel.prev_msg) then
    --debug_message("g_game_msg: " .. g_game_msg);
        local l_trade_index = l_message:find(g_message_trade_uid, 1, true)
        if (l_trade_index ~= nil) then
            --print("trade message saved.")
            --debug_message(sprintf("l_trade_index: %d", l_trade_index));
            local l_log_msg = l_message:sub((g_message_trade_uid:len() + 1))
            local l_rec = get_dt_rec(l_log_msg)
            append_to_file(channel.log_file, l_rec)
            channel.prev_msg = l_message
        end
    end    
end


function save_common_message(proc, channel)
    if (not g_cfg.write_logs) then
        return
    end
    local l_message = Hook:get_latest_text_msg(channel.name, proc)
    if (l_message ~= nil and l_message ~= channel.prev_msg) then
        --print("common message saved.")
        local l_rec = get_dt_rec(l_message)
        append_to_file(channel.log_file, l_rec)
        channel.prev_msg = l_message
    end
end


-- Loads the Shaiya config.
-- @returns the config array.
function load_config(game_path)
  local l_config = {}
  local l_file = assert(io.open(game_path .. "CONFIG.INI", "r"))
  
  for line in l_file:lines() do
    --debug_message(line)
    if (string.find(line, "%[(.*)%]") ~= nil) then
      local l_group = string.sub(line,
        (string.find(line, "%[") + 1), 
        (string.find(line, "%]") - 1))
      --debug_message("group: " .. l_group)
      l_config[l_group] = {}
      for config in l_file:lines() do
        if (config == "") then
          break;
        end
        local l_delim = string.find(config, "=")
        if (l_delim ~= nil) then
          local l_key = string.sub(config, 1, (l_delim-1))
          local l_val = string.sub(config, (l_delim+1))
          l_config[l_group][l_key] = l_val
          --debug_message("l_config." .. l_group .. "." .. l_key .. "=" .. l_config[l_group][l_key])
        end
      end
    end
  end
  return l_config
end

function pass_arguments()
    -- Can pass addditional arguments here.
    -- arg -cfg pass a custom config xml file in ".\data\".
    -- arg -char pass a custom character xml file in ".\char\".
    local l_args_func = {
       ["-cfg"] = function(val) g_xml_file_config = g_script_dir .. "data/" .. val; end,
       ["-char"] = function(val) g_xml_file_char = g_script_dir .. "char/" .. val; end,
    }
    for k,v in pairs(args) do
        if (type(l_args_func[v]) == "function") then
            l_args_func[v](args[(k + 1)])
        end
    end
end
