-- merge: Merge two tables
-- If there are duplicate fields, u's will be used.
--   t, u: tables
-- returns
--   r: the merged table
function merge(t, u)
  local r = {}
  for k,v in pairs(t) do
    r[k] = v
  end
  for k,v in pairs(u) do
    r[k] = v
  end
  --table.foreach(t, function (i, v) r[i] = v end)
  --table.foreach(u, function (i, v) r[i] = v end)
  return r
end


function debug_message(msg)
  if (g_debug) then
    print(msg)
    logMessage(msg)
  end
end


function distance_3d_point(point_a, point_b)
  --local l_d = Vertex()
    local l_d = {
        x=(point_a.x - point_b.x),
        y=(point_a.y - point_b.y),
        z=(point_a.z - point_b.z),
    };

  --debug_message(sprintf("l_d: %f, %f, %f", l_d.x, l_d.y, l_d.z));
  return ((l_d.x^2) + (l_d.y^2) + (l_d.z^2)) ^ 0.5
end


function distance_2d_point(point_a, point_b)
    --local l_d = Vertex();
    --[[print("point_a")
    print_r(point_a)
    print("point_b")
    print_r(point_b)]]
    local l_d = {
        x=(point_a.x - point_b.x),
        y=(point_a.y - point_b.y),
    }

    --debug_message(sprintf("l_d: %f, %f, %f", l_d.x, l_d.y, l_d.z))
    return ((l_d.x^2) + (l_d.y^2)) ^ 0.5
end


function table_len(t)
  local i = 0
  for k,v in pairs(t) do
    i = i + 1
  end
  return i
end


function get_dt_rec(data)
  local l_dt = os.date(("[%x %X ".. g_timezone .. "] "), os.time())
  return (l_dt .. data .. "\n")
end


function append_to_file(file_name, data)
  local l_file = assert(io.open(file_name, "a"))
    l_file:write(data)
  l_file:close()
end


-- looks for the needle in the file haystack.
-- @param file_path path and name of file to open.
-- @param needle the needle you want to search for in file.
-- @return file line on success, false otherwise.
function find_in_file(file_path, needle)
  local l_file = assert(io.open(file_path, "r"))
  local l_result = false;
  for line in l_file:lines() do
    if (line:find(needle)) then
      l_result = line;
      break;
    end
  end
  l_file:close();
  return l_result;
end


-- matches a pixel to a color threshold table.
-- return true if in threshold, false otherwise.
function match_pixel(rgb_val, rgb_min, rgb_max)
  --rgb_val:debug_msg()
  for k,v in pairs(rgb_val) do
    --debug_message("red: " .. rgb_min["red"])
    if (v < rgb_min[k] or v > rgb_max[k]) then
      return false
    end
  end
  return true
end

-- matches pixels in a rectangle.
-- @param hdc handle device context.
-- @param color The color you want to search for.
-- @param point_begin The x,y coordinates of the start point.
-- @param point_end The x,y coords of end point.
-- @param accuracy all color channels must be within 'accuracy' of given color.
-- @return results A table of x,y coordinates of matches, otherwise nil.
function match_pixels_in_rect(hdc, color, point_begin, point_end, accuracy)
    local x = os.clock()
    local s = 0

    results = {}
    accuracy = accuracy or 0
    print_r(color)
    print_r(point_begin)
    print_r(point_end)
    print_r(accuracy)
    
    local l_clr_channel = {
        r=getR(color),
        g=getG(color),
        b=getB(color)
    }
    
    local l_r, l_g, l_b
    for i=point_begin[1], point_end[1] do
        for j=point_begin[2], point_end[2] do
            l_r, l_g, l_b = getPixel(hdc, i, j)
            --local l_pixel_color = makeColor(getPixel(hdc, i, j))
            if(math.abs(l_clr_channel.r - l_r) <= accuracy and
                    math.abs(l_clr_channel.g - l_g) <= accuracy  and
                    math.abs(l_clr_channel.b - l_b) <= accuracy) then
                table.insert(results, {x=i, y=j})
            end
            coroutine.yield()
        end
    end

    print(string.format("elapsed time: %.2f\n", os.clock() - x))    
    return results
end


function get_quadrantal_angle(pos)
    -- find out the opposite angle of the right sided triangle.
    local l_deg_cur = 90 - (math.deg(math.atan(pos.x / pos.y)))
    --debug_message("l_deg_cur: " .. l_deg_cur)
    -- If it's in the 3rd or 4th Quadrant then add 180 degrees.
    if (get_quadrant(pos) >= 3) then
        l_deg_cur = (l_deg_cur + 180) % 360
    end
    --return round(l_deg_cur, 1) % 360
    return l_deg_cur 
end


function get_quadrant(pos)
    if (pos.x >= 0) then 
        if (pos.y >= 0) then
            return 1
        else
            return 4
        end
    else
        if (pos.y >= 0) then
            return 2
        else
            return 3
        end
    end
end


--[[
  Get the closest rotation direction.
  @param degree The current degrees that object is facing (0-360).
  @param degree_new The new degree that object wants to face (0-360).
  @return Rotation direction, DIRECTION_RIGHT for ClockWise (CW), DIRECTION_LEFT for CCW.
]]--
function get_rotate_direction(degree, degree_new)
    l_d_rad = math.rad(degree)
    l_d_new_rad = math.rad(degree_new)
    local l_diff = math.atan2(math.sin(l_d_new_rad-l_d_rad),math.cos(l_d_new_rad-l_d_rad));
    --debug_message("l_diff="..l_diff)
    if (l_diff >= 0) then
        return DIRECTION_LEFT
    else
        return DIRECTION_RIGHT
    end
end


function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function print_r ( t ) 
 	local print_r_cache={}
	local function sub_print_r(t,indent)
		if (print_r_cache[tostring(t)]) then
			print(indent.."*"..tostring(t))
		else
			print_r_cache[tostring(t)]=true
			if (type(t)=="table") then
				for pos,val in pairs(t) do
					if (type(val)=="table") then
						print(indent.."["..pos.."] => "..tostring(t).." {")
						sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
						print(indent..string.rep(" ",string.len(pos)+6).."}")
					else
						print(indent.."["..pos.."] => "..tostring(val))
					end
				end
			else
				print(indent..tostring(t))
			end
		end
	end
	sub_print_r(t,"  ")
end

function is_number(data)
    --print("data: " .. data)
    local l_res = string.find(data, "[^%.?0-9+]")
    if (l_res ~= nil) then
        --print("not number")
        return false
    else
        --print("number")
        return true
    end
end

function exists(fname)
    local f = io.open(fname, "r")
    if (f and f:read()) then
        return true
    end
end


-- explicit string conversion to another basic data type.
-- currently only number and boolean types supported.
-- @param data The data you want to convert.
-- @param data_type The basic data type you want to change it too.
-- @return The converted data
function stringConvert(data, data_type)
    converts = {}
    converts["boolean"] = function()
        if (data == "true") then
            return true
        elseif (data == "false") then
            return false
        else
            return nil
        end
    end
    converts["number"] = function()
        print("data: " .. data)
        local l_res = string.find(data, "[^%.?0-9+]")
        if (l_res == nil) then
            print("number")
            return tonumber(data)
        else
            print("not number")
            return nil
        end   
    end
    print("data: " .. data)
    print("data_type: " .. data_type)
    return converts[data_type]()
end

-- implicit string conversion to another basic data type.
-- currently only number and boolean types supported.
-- @param data The data you want to convert.
-- @return The converted data.
function string_convert(data)
    print("data: " .. data)
    if (data == "true") then
        return true
    elseif (data == "false") then
        return false
    elseif(string.find(data, "[^%.?0-9+]") == nil) then
        return tonumber(data)
    else
        return data
    end
end


function mouseSetWin(win, x, y)
  local l_x, l_y, l_w, l_h = windowRect(win)
  mouseSet(l_x + x, l_y + y)  
end

function setStartKey(val)
    startKey = val
end

function getStartKey()
    return startKey
end

function setStopKey(val)
    stopKey = val
end

function getStopKey()
    return stopKey
end


-- A pretty standard rest/sleep command.
-- It automatically will yeild, though.
function yrest(msec)
  if( msec == nil ) then error("yrest() cannot rest for \'nil\'.\n", 2); end;
  safeYield();

  -- if Coco is available, use 10ms sections. this is the highest
  -- resolution available on Windows.
  -- if Coco is not available, fall back on 100ms sections to
  -- avoid yielding across C-boundaries

  local resttime;
  local sections;
  local ext;

  if( cocoAvailable ) then
    resttime = 50;
  else
    resttime = 100;
  end


  if( msec < resttime ) then
    rest(msec);

    return;
  else
    sections = math.floor(msec / resttime); -- split into 10/100ms sections
    ext = math.mod(msec, resttime); -- any left overs...

    for b = 1,sections do
      rest(resttime);
      safeYield();
    end;

    if( ext > 0 ) then
      rest(ext);
    end
  end
end
